#!/usr/bin/env python3
# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import re
import sys
from typing import Optional, Tuple

import click
from aws_ddk.__metadata__ import __version__
from aws_ddk.commands.bootstrap import bootstrap_account
from aws_ddk.commands.create import create_code_repository
from aws_ddk.commands.deploy import cdk_deploy
from aws_ddk.commands.init import init_project
from aws_ddk.utils import get_package_root
from boto3 import Session, _get_default_session, setup_default_session

DEBUG_LOGGING_FORMAT = "[%(asctime)s][%(filename)-13s:%(lineno)3d] %(message)s"
DEBUG_LOGGING_FORMAT_REMOTE = "[%(filename)-13s:%(lineno)3d] %(message)s"
DEFAULT_PROJECT_TEMPLATE = "data/project_templates/ddk_app/"

_logger: logging.Logger = logging.getLogger(__name__)


def enable_debug(format: str) -> None:
    logging.basicConfig(level=logging.DEBUG, format=format)
    _logger.setLevel(logging.DEBUG)
    logging.getLogger("boto3").setLevel(logging.ERROR)
    logging.getLogger("botocore").setLevel(logging.ERROR)
    logging.getLogger("urllib3").setLevel(logging.ERROR)
    logging.getLogger("cookiecutter").setLevel(logging.ERROR)
    logging.getLogger("sh").setLevel(logging.ERROR)


def setup_boto_session(profile: str, region: Optional[str] = None) -> None:
    # Setup profile and region globally in boto3 sessions
    setup_default_session(profile_name=profile, region_name=region)
    session: Session = _get_default_session()
    _logger.debug(f"profile: {session.profile_name}")
    _logger.debug(f"region: {session.region_name}")


class RegexString(click.ParamType):
    name = "regex-string"

    def __init__(self, regex: str):
        self._regex = regex

    def convert(self, value: str, param: Optional[click.Parameter], ctx: Optional[click.Context]) -> str:
        found = re.match(self._regex, value)
        if not found:
            self.fail(
                f"Value '{value}' does not match regex {self._regex}",
                param,
                ctx,
            )
        return value


@click.group()
@click.version_option(__version__)
@click.option(
    "--debug/--no-debug",
    default=False,
    help="Turn debug logging on/off.",
    show_default=True,
)
def cli(
    debug: bool,
) -> None:
    """AWS DDK CLI."""
    click.echo(f"AWS DDK CLI {__version__} (Python {sys.version})")
    if debug:
        enable_debug(format=DEBUG_LOGGING_FORMAT)
    _logger.debug(f"debug: {debug}")


@cli.command(name="init")
@click.argument(
    "name",
    type=str,
    required=True,
)
@click.option(
    "--environment",
    "-e",
    type=str,
    help="The id of the environment.",
    required=True,
    default="dev",
    show_default=True,
)
@click.option(
    "--template",
    "-t",
    type=str,
    help="A directory containing a project template directory, or a URL to a git repository",
)
@click.option(
    "--generate-only",
    is_flag=True,
    type=bool,
    help="If true, only generates project files, without setting up a git repo or a virtual environment",
    default=False,
    show_default=True,
)
def init(name: str, environment: str, template: Optional[str] = None, generate_only: Optional[bool] = None) -> None:
    """
    Create the local structure for a new AWS DDK Python project.

    NAME is the name of the project.
    """
    # Use default Cookiecutter project template
    if not template:
        template = os.path.join(get_package_root(), DEFAULT_PROJECT_TEMPLATE)
    return init_project(name=name, environment=environment, template=template, generate_only=generate_only)


@cli.command(name="bootstrap")
@click.option(
    "--environment",
    "-e",
    type=RegexString(regex=r"^[A-Za-z0-9_-]{1,4}$"),
    help="The id of the environment.",
    required=True,
    default="dev",
    show_default=True,
)
@click.option(
    "--profile",
    "-p",
    type=str,
    default="default",
    help="Use a specific profile from your AWS credentials file.",
    show_default=True,
    required=False,
)
@click.option(
    "--region",
    "-r",
    type=str,
    default=None,
    help="AWS Region name (e.g. us-east-1). If None, it will be inferred.",
    show_default=False,
    required=False,
)
@click.option(
    "--prefix",
    type=RegexString(regex=r"^[A-Za-z0-9_-]{1,5}$"),
    help="The prefix to resource names.",
    required=False,
    default="ddk",
    show_default=True,
)
@click.option(
    "--qualifier",
    type=RegexString(regex=r"^[A-Za-z0-9_-]{1,10}$"),
    help="The CDK bootstrap qualifier.",
    required=False,
)
@click.option(
    "--trusted-accounts",
    "-a",
    type=str,
    help="List of trusted AWS accounts to perform deployments (e.g. -a 111111111111 -a 222222222222).",
    multiple=True,
    required=False,
)
@click.option(
    "--iam-policies",
    "-i",
    type=str,
    help="""List of IAM managed policy ARNs that should be attached to the role performing deployments.
    (e.g. -i arn1 -i arn2)""",
    multiple=True,
    required=False,
)
@click.option(
    "--permissions-boundary",
    type=str,
    help="IAM managed permissions boundary policy ARN that should be attached to the role performing deployments.",
    required=False,
)
@click.option(
    "--tags",
    "-t",
    type=(str, str),
    help="List of tags to apply to the stack (e.g -t CostCenter 1984 -t Framework DDK).",
    multiple=True,
    required=False,
)
def bootstrap(
    environment: str,
    profile: str,
    region: Optional[str] = None,
    prefix: Optional[str] = None,
    qualifier: Optional[str] = None,
    trusted_accounts: Optional[Tuple[str]] = None,
    iam_policies: Optional[Tuple[str]] = None,
    permissions_boundary: Optional[str] = None,
    tags: Optional[Tuple[Tuple[str, str]]] = None,
) -> None:
    """Bootstrap the AWS account with DDK resources."""
    setup_boto_session(profile, region)
    bootstrap_account(
        environment=environment,
        prefix=prefix,
        qualifier=qualifier,
        trusted_accounts=trusted_accounts,
        iam_policies=iam_policies,
        permissions_boundary=permissions_boundary,
        tags=tags,
    )


@cli.command(name="create-repository")
@click.argument(
    "name",
    type=str,
    required=True,
)
@click.option(
    "--profile",
    "-p",
    type=str,
    default="default",
    help="Use a specific profile from your AWS credentials file.",
    show_default=True,
    required=False,
)
@click.option(
    "--region",
    "-r",
    type=str,
    default=None,
    help="AWS Region name (e.g. us-east-1). If None, it will be inferred.",
    show_default=False,
    required=False,
)
@click.option(
    "--description",
    "-d",
    type=str,
    help="The description of the repository.",
    required=False,
)
@click.option(
    "--tags",
    "-t",
    type=(str, str),
    help="List of tags to apply to the repository (e.g -t CostCenter 1984 -t Framework DDK).",
    multiple=True,
    required=False,
)
def create_repository(
    name: str,
    profile: str,
    region: Optional[str] = None,
    description: Optional[str] = None,
    tags: Optional[Tuple[Tuple[str, str]]] = None,
) -> None:
    """
    Create a code repository from the source system provider.

    NAME is the name of the repository.
    """
    setup_boto_session(profile, region)
    create_code_repository(
        profile=profile,
        name=name,
        description=description,
        tags=tags,
    )


@cli.command(name="deploy")
@click.option(
    "--profile",
    "-p",
    type=str,
    default="default",
    help="Use a specific profile from your AWS credentials file.",
    show_default=True,
    required=False,
)
@click.option(
    "--require-approval",
    type=click.Choice(["never", "any-change", "broadening"], case_sensitive=False),
    default="never",
    help="What security-sensitive changes need manual approval.",
    required=False,
)
@click.option(
    "--force",
    "-f",
    is_flag=True,
    default=False,
    help="Always deploy stack even if templates are identical.",
    required=False,
)
@click.option(
    "--output-dir",
    "-o",
    type=str,
    help="Directory where cloud assembly is synthesized.",
    required=False,
)
def deploy(
    profile: str,
    require_approval: Optional[str] = None,
    force: Optional[bool] = None,
    output_dir: Optional[str] = None,
) -> None:
    """Deploy DDK stacks to AWS account."""
    setup_boto_session(profile)
    cdk_deploy(
        profile=profile,
        require_approval=require_approval,
        force=force,
        output_dir=output_dir,
    )


def main() -> int:
    cli()
    return 0



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Dict, Optional, Tuple

from aws_ddk.services import codecommit
from aws_ddk.sh import run
from aws_ddk.utils import is_in_git_repository
from click import echo, secho

_logger: logging.Logger = logging.getLogger(__name__)


def tuples_to_dict(tuples: Tuple[Tuple[str, str]]) -> Dict[str, str]:
    dict: Dict[str, str] = {}
    for k, v in tuples:
        dict.setdefault(k, v)
    return dict


def create_code_repository(
    profile: str,
    name: str,
    description: Optional[str] = None,
    tags: Optional[Tuple[Tuple[str, str]]] = None,
) -> None:
    _logger.debug(f"name: {name}")

    echo("Creating AWS CodeCommit repository...")
    dict_tags = tuples_to_dict(tags) if tags else None
    url = codecommit.create_repository(
        name=name,
        description=description,
        tags=dict_tags,
    )

    echo("Running git commands...")
    path = "."
    if is_in_git_repository(path):
        cmds = [
            f'git config --local credential.helper "!aws codecommit --profile {profile} credential-helper $@"',
            "git config --local credential.UseHttpPath true",
            f"git remote add origin {url}",
        ]
        try:
            for cmd in cmds:
                run(cmd, path)
        except Exception:
            secho(f"Failed to run `{cmd}`", blink=True, bold=True, fg="red")
    else:
        raise SystemExit("The current working directory does not contain a .git directory.")
    echo("Done.")



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
import time
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple

import botocore.exceptions
from aws_ddk.utils import boto3_client

if TYPE_CHECKING:
    from mypy_boto3_cloudformation.type_defs import WaiterConfigTypeDef

CHANGESET_PREFIX = "aws-ddk-cli-deploy-"

_logger: logging.Logger = logging.getLogger(__name__)


def get_stack_status(stack_name: str) -> str:
    client = boto3_client("cloudformation")
    try:
        resp = client.describe_stacks(StackName=stack_name)
        if len(resp["Stacks"]) < 1:
            raise ValueError(f"CloudFormation stack {stack_name} not found.")
    except botocore.exceptions.ClientError:
        raise
    return resp["Stacks"][0]["StackStatus"]


def does_stack_exist(stack_name: str) -> bool:
    client = boto3_client("cloudformation")
    try:
        resp = client.describe_stacks(StackName=stack_name)
        if len(resp["Stacks"]) < 1:
            return False
    except botocore.exceptions.ClientError as ex:
        error = ex.response["Error"]
        if error["Code"] == "ValidationError" and f"Stack with id {stack_name} does not exist" in error["Message"]:
            return False
        raise
    return True


def _wait_for_changeset(changeset_id: str, stack_name: str) -> bool:
    waiter = boto3_client("cloudformation").get_waiter("change_set_create_complete")
    waiter_config: "WaiterConfigTypeDef" = {"Delay": 1}
    try:
        waiter.wait(ChangeSetName=changeset_id, StackName=stack_name, WaiterConfig=waiter_config)
    except botocore.exceptions.WaiterError as ex:
        resp = ex.last_response
        status = resp["Status"]
        reason = resp["StatusReason"]
        if (
            status == "FAILED"
            and "The submitted information didn't contain changes." in reason
            or "No updates are to be performed" in reason
        ):
            _logger.debug(f"No changes to {stack_name} CloudFormation stack.")
            return False
        raise RuntimeError(f"Failed to create the changeset: {ex}. Status: {status}. Reason: {reason}")
    return True


def _create_changeset(
    stack_name: str,
    template_str: str,
    parameters: Optional[List[Dict[str, str]]] = None,
    tags: Optional[List[Dict[str, str]]] = None,
) -> Tuple[str, str]:
    now = str(int(time.time()))
    description = f"Created by AWS DDK CLI at {now} UTC"
    changeset_name = CHANGESET_PREFIX + now
    changeset_type = "UPDATE" if does_stack_exist(stack_name=stack_name) else "CREATE"
    kwargs: Dict[str, Any] = {
        "ChangeSetName": changeset_name,
        "StackName": stack_name,
        "ChangeSetType": changeset_type,
        "Capabilities": ["CAPABILITY_IAM", "CAPABILITY_NAMED_IAM"],
        "Description": description,
        "TemplateBody": template_str,
    }
    if parameters:
        kwargs["Parameters"] = parameters
    if tags:
        kwargs["Tags"] = tags
    resp = boto3_client("cloudformation").create_change_set(**kwargs)
    return str(resp["Id"]), changeset_type


def _execute_changeset(changeset_id: str, stack_name: str) -> None:
    boto3_client("cloudformation").execute_change_set(ChangeSetName=changeset_id, StackName=stack_name)


def _wait_for_execute(stack_name: str, changeset_type: str) -> None:
    waiter_config: "WaiterConfigTypeDef" = {
        "Delay": 5,
        "MaxAttempts": 480,
    }

    if changeset_type == "CREATE":
        create_waiter = boto3_client("cloudformation").get_waiter("stack_create_complete")
        create_waiter.wait(StackName=stack_name, WaiterConfig=waiter_config)
        return

    if changeset_type == "UPDATE":
        update_waiter = boto3_client("cloudformation").get_waiter("stack_update_complete")
        update_waiter.wait(StackName=stack_name, WaiterConfig=waiter_config)
        return

    raise RuntimeError(f"Invalid changeset type {changeset_type}")


def deploy_template(
    stack_name: str,
    file_name: str,
    parameters: Optional[List[Dict[str, str]]] = None,
    tags: Optional[List[Dict[str, str]]] = None,
) -> None:
    _logger.debug("Deploying template %s", file_name)
    if not os.path.isfile(file_name):
        raise FileNotFoundError(f"CloudFormation template not found at {file_name}")
    with open(file_name, "r") as handle:
        template_str = handle.read()
    changeset_id, changeset_type = _create_changeset(
        stack_name=stack_name, template_str=template_str, parameters=parameters, tags=tags
    )
    has_changes = _wait_for_changeset(changeset_id, stack_name)
    if has_changes:
        _execute_changeset(changeset_id=changeset_id, stack_name=stack_name)
        _wait_for_execute(stack_name=stack_name, changeset_type=changeset_type)


def destroy_stack(stack_name: str) -> None:
    _logger.debug("Destroying stack %s", stack_name)
    client = boto3_client("cloudformation")
    client.delete_stack(StackName=stack_name)
    waiter = client.get_waiter("stack_delete_complete")
    waiter.wait(StackName=stack_name, WaiterConfig={"Delay": 5, "MaxAttempts": 200})



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Optional

from aws_ddk.sh import run
from aws_ddk.utils import get_account_id, get_region
from click import echo

_logger: logging.Logger = logging.getLogger(__name__)


def cdk_deploy(
    profile: str,
    require_approval: Optional[str] = None,
    force: Optional[bool] = None,
    output_dir: Optional[str] = None,
) -> None:

    echo(f"Deploying DDK stacks to AWS account {get_account_id()} and region {get_region()}...")

    # generate command
    cmd = (
        "cdk deploy "
        f"--profile {profile} "
        f"{'--require-approval ' + require_approval + ' ' if require_approval else ''}"
        f"{'-f ' if force else ''}"
        f"--output {output_dir if output_dir else '.ddk.out'}"
    )

    try:
        run(cmd)
    except Exception:
        raise SystemExit(f"ERROR - Failed to run `{cmd}`. See output above.")
    echo("Done.")



#!/usr/bin/env python3

import aws_cdk as cdk
from {{cookiecutter.package_name}}.{{cookiecutter.stack_file_name}} import {{cookiecutter.stack_name}}

app = cdk.App()
{{cookiecutter.stack_name}}(app, "{{cookiecutter.stack_name}}", "{{cookiecutter.environment_id}}")

app.synth()



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
from typing import Dict, Optional

import botocore.exceptions
from aws_ddk.utils import boto3_client
from click import echo

_logger: logging.Logger = logging.getLogger(__name__)


def create_repository(name: str, description: Optional[str], tags: Optional[Dict[str, str]]) -> str:
    client = boto3_client("codecommit")
    try:
        resp = client.create_repository(
            repositoryName=name,
            repositoryDescription=description or "",
            tags=tags or {},
        )
    except botocore.exceptions.ClientError as ex:
        error = ex.response["Error"]
        if error["Code"] == "RepositoryNameExistsException":
            echo(f"Repository {name} already exists in the AWS account.")
            resp = client.get_repository(repositoryName=name)
        else:
            raise ex
    return resp["repositoryMetadata"]["cloneUrlHttp"]



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import os
from typing import Dict, List, Optional, Tuple

from aws_ddk.services import cfn
from aws_ddk.utils import get_account_id, get_package_root, get_region
from click import echo

DEFAULT_BOOTSTRAP_TEMPLATE = "data/cloudformation_templates/bootstrap.yaml"
PREFIX = "ddk"

_logger: logging.Logger = logging.getLogger(__name__)


def tuples_to_list(tuples: Tuple[Tuple[str, str]]) -> List[Dict[str, str]]:
    return [{"Key": k, "Value": v} for k, v in tuples]


def bootstrap_account(
    environment: str,
    prefix: Optional[str] = None,
    qualifier: Optional[str] = None,
    trusted_accounts: Optional[Tuple[str]] = None,
    iam_policies: Optional[Tuple[str]] = None,
    permissions_boundary: Optional[str] = None,
    tags: Optional[Tuple[Tuple[str, str]]] = None,
) -> None:
    _logger.debug(f"environment: {environment}")

    if not prefix:
        prefix = PREFIX

    parameters = [
        {
            "ParameterKey": "Prefix",
            "ParameterValue": prefix,
        },
        {
            "ParameterKey": "Environment",
            "ParameterValue": environment,
        },
    ]

    if qualifier:
        parameters.append({"ParameterKey": "Qualifier", "ParameterValue": qualifier})
    if trusted_accounts:
        parameters.append({"ParameterKey": "TrustedAccounts", "ParameterValue": ",".join(trusted_accounts)})
    if iam_policies:
        parameters.append({"ParameterKey": "CloudFormationExecutionPolicies", "ParameterValue": ",".join(iam_policies)})
    else:
        echo("No IAM policy supplied. Defaulting to Admin permissions.")
    if permissions_boundary:
        parameters.append(
            {
                "ParameterKey": "CloudFormationExecutionPermissionsBoundaryPolicy",
                "ParameterValue": permissions_boundary,
            }
        )
    key_value_tags = tuples_to_list(tags) if tags else None

    echo(f"Bootstrapping environment {environment} - AWS account {get_account_id()} and region {get_region()}...")
    _logger.debug(f"parameters: {parameters}")
    cfn.deploy_template(
        stack_name=f"{prefix.title()}{environment.title()}Bootstrap",
        file_name=os.path.join(get_package_root(), DEFAULT_BOOTSTRAP_TEMPLATE),
        parameters=parameters,
        tags=key_value_tags,
    )
    echo("Done.")



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from pathlib import Path
from typing import TYPE_CHECKING, overload

import boto3
from aws_ddk.__metadata__ import __version__
from botocore.config import Config

if TYPE_CHECKING:
    from boto3.resources.base import ServiceResource
    from botocore.client import BaseClient
    from mypy_boto3_cloudformation.client import CloudFormationClient
    from mypy_boto3_codecommit.client import CodeCommitClient
    from mypy_boto3_codecommit.literals import ServiceName
    from mypy_boto3_sts.client import STSClient
    from typing_extensions import Literal


def get_botocore_config() -> Config:
    return Config(
        retries={"max_attempts": 5},
        connect_timeout=10,
        max_pool_connections=10,
        user_agent_extra=f"awsddk/{__version__}",
    )


@overload
def boto3_client(service_name: 'Literal["sts"]') -> "STSClient":
    ...


@overload
def boto3_client(service_name: 'Literal["cloudformation"]') -> "CloudFormationClient":
    ...


@overload
def boto3_client(service_name: 'Literal["codecommit"]') -> "CodeCommitClient":
    ...


def boto3_client(service_name: "ServiceName") -> "BaseClient":
    return boto3.client(service_name=service_name, use_ssl=True, config=get_botocore_config())


def boto3_resource(service_name: "ServiceName") -> "ServiceResource":
    return boto3.resource(service_name=service_name, use_ssl=True, config=get_botocore_config())


def get_region() -> str:
    session = boto3._get_default_session()
    if session.region_name is None:
        raise ValueError("It is not possible to infer AWS REGION from your environment.")
    return str(session.region_name)


def get_account_id() -> str:
    return str(boto3_client(service_name="sts").get_caller_identity().get("Account"))


def get_package_root() -> str:
    return os.path.dirname(os.path.relpath(__file__))


def is_in_git_repository(path: str) -> bool:
    return Path(path, ".git").is_dir()



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Metadata Module.

Source repository: https://github.com/awslabs/aws-ddk
Documentation: TBD
"""

__version__: str = "0.2.0"
__title__: str = "aws_ddk"
__description__: str = "AWS DataOps Development Kit - CLI"
__license__: str = "Apache-2.0"



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import shlex
import subprocess
from typing import Iterable, Optional

from aws_ddk.exceptions import FailedShellCommand
from click import echo

_logger: logging.Logger = logging.getLogger(__name__)


def _clean_up_stdout_line(line: bytes) -> str:
    line_str = line.decode("utf-8")
    return line_str[:-1] if line_str.endswith("\n") else line_str


def _run_iterating(cmd: str, cwd: Optional[str] = None) -> Iterable[str]:
    p = subprocess.Popen(shlex.split(cmd), cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    if p.stdout is None:
        return []
    while p.poll() is None:
        yield _clean_up_stdout_line(line=p.stdout.readline())
    if p.returncode != 0:
        raise FailedShellCommand(f"Exit code: {p.returncode}")


def run(cmd: str, cwd: Optional[str] = None, hide_cmd: bool = False) -> None:
    if hide_cmd is False:
        _logger.debug(f"+ {cmd}")
    for line in _run_iterating(cmd=cmd, cwd=cwd):
        echo(line)



from typing import Any

from aws_ddk_core.base import BaseStack
from aws_ddk_core.resources import S3Factory
from constructs import Construct


class {{cookiecutter.stack_name}}(BaseStack):

    def __init__(self, scope: Construct, id: str, environment_id: str, **kwargs: Any) -> None:
        super().__init__(scope, id, environment_id, **kwargs)

        # The code that defines your stack goes here. For example:
        ddk_bucket = S3Factory.bucket(
            self,
            "ddk-bucket",
            environment_id,
        )



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging
import sys
from typing import Optional

from aws_ddk.sh import run
from aws_ddk.utils import is_in_git_repository
from click import echo, secho
from cookiecutter.main import cookiecutter

_logger: logging.Logger = logging.getLogger(__name__)


def python_executable() -> str:
    if sys.platform == "win32":
        return "python"
    return "python3"


def init_project(name: str, environment: str, template: Optional[str], generate_only: Optional[bool]) -> None:
    _logger.debug(f"name: {name}")
    _logger.debug(f"environment: {environment}")
    _logger.debug(f"template: {template}")
    _logger.debug(f"generate_only: {generate_only}")

    python_exec: str = python_executable()

    # Initialize from a project template
    echo("Initializing AWS DDK project...")
    path: str = cookiecutter(
        template,
        no_input=True,
        extra_context={
            "directory_name": name,
            "environment_id": environment,
            "python_executable": python_exec,
        },
    )

    if not generate_only:
        # Create git repository
        if not is_in_git_repository(path):
            echo("Initializing a new git repository...")
            cmds = [
                "git init",
                "git checkout -b main",
                "git add .",
                "git commit --message='Initial commit' --no-gpg-sign",
            ]
            try:
                for cmd in cmds:
                    run(cmd, path)
            except Exception:
                secho(f"Failed to run `{cmd}`", blink=True, bold=True, fg="red")

        # Create virtual environment (.venv)
        echo(f"Creating virtual environment in `{path}`...")
        cmd = f"{python_exec} -m venv '{path}/.venv'"
        try:
            run(cmd)
        except Exception:
            secho(f"Failed to run `{cmd}`", blink=True, bold=True, fg="red")

    echo("Done.")



import setuptools

with open("README.md") as fp:
    long_description = fp.read()


setuptools.setup(
    name="{{cookiecutter.directory_name}}",
    version="{{cookiecutter.ddk_version}}",
    description="An empty DDK Python app",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="author",
    package_dir={"": "{{cookiecutter.directory_name}}"},
    packages=setuptools.find_packages(where="{{cookiecutter.directory_name}}"),
    install_requires=open("requirements.txt").read().strip().split("\n"),
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Programming Language :: JavaScript",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Software Development :: Code Generators",
        "Topic :: Utilities",
        "Typing :: Typed",
    ],
)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['aws_ddk',
 'aws_ddk.commands',
 'aws_ddk.data.project_templates.ddk_app.{{cookiecutter.directory_name}}',
 'aws_ddk.data.project_templates.ddk_app.{{cookiecutter.directory_name}}.{{cookiecutter.package_name}}',
 'aws_ddk.services']

package_data = \
{'': ['*'],
 'aws_ddk': ['data/cloudformation_templates/*',
             'data/project_templates/ddk_app/*']}

install_requires = \
['boto3>=1.20.37,<2.0.0',
 'botocore>=1.23.37,<2.0.0',
 'click>=8.0.3,<9.0.0',
 'cookiecutter>=1.7.3,<2.0.0']

entry_points = \
{'console_scripts': ['ddk = aws_ddk.__main__:main']}

setup_kwargs = {
    'name': 'aws-ddk',
    'version': '0.2.0',
    'description': 'AWS DataOps Development Kit - CLI',
    'long_description': '# AWS DataOps Development Kit (DDK) CLI\n\nCommand line interface to manage your DDK Apps.',
    'author': 'AWS Professional Services',
    'author_email': 'aws-proserve-opensource@amazon.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/awslabs/aws-ddk',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
